@@ -33,21 +33,6 @@ pub fn generate_random_number() -> Option<u32> {
 	None
 }
 
-/// Search the most significant bit
-#[inline(always)]
-pub fn msb(value: u64) -> Option<u64> {
-	if value > 0 {
-		let ret: u64;
-		let u64_bits = 64;
-		unsafe {
-			asm!("clz $0, $1; sub $0, $2, $0" : "=r"(ret) : "r"(value), "r"(u64_bits - 1) : "cc" : "volatile");
-		}
-		Some(ret)
-	} else {
-		None
-	}
-}
-
 /// The halt function stops the processor until the next interrupt arrives
 pub fn halt() {
 	unsafe {
